/*  -----------------------------------------------------------------------
    tish - unixlike shell & tools for the TI-92+ and Voyage 200 calculators
    Copyright (C) 2020 Adam Colley <mog@kupo.be>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
------------------------------------------------------------------------ */

// help command
// this is part of tish

#define USE_TI92PLUS
#define USE_V200

#include <tigcclib.h>

void _main(void)
{
  ESI argptr = top_estack;
  int args,c;
  char page[64];
  
  args=RemainingArgCnt(argptr);
  c=args;

  while (c)
  {
    char *a = GetStrnArg (argptr);
    sprintf(page,"%s",a);
    c--;
    top_estack = next_expression_index (top_estack);
  }

  if (args==0)
  {
    printf("tish - unixlike shell for TI 92+/Voy200\n\n");    printf("Path: internal, current, \\tish, \\bin\n\n");
    printf("Internal tish commands:\n");
    printf(" # cd clear echo exit off pause\n\n");
    printf("External tish commands:\n");
    printf(" arc cp help ls mkdir mv rm rmdir unarc\n\n");
    printf("Use help <command> for additional help\n");
  }
  else if (!strcmp(page,"arc"))
  {
    printf("arc - move file to archive storage\n\n");
    printf("arc <filename>\n");
  }
  else if (!strcmp(page,"cd"))
  {
    printf("cd - Change directory (folder)\n\n");
    printf("cd <foldername>\n\n");
    printf("main assumed if foldername omitted\n");
  }
  else if (!strcmp(page,"clear"))
  {
    printf("clear - Clear screen\n\n");
    printf("clear\n");
  }
  else if (!strcmp(page,"cp"))
  {
    printf("cp - Copy file\n\n");
    printf("cp <source> <destination>\n\n");
    printf("Note: copied file will be in RAM\n");
  }
  else if (!strcmp(page,"echo"))
  {
    printf("echo - Display text message\n\n");
    printf("echo [text]\n");
  }
  else if (!strcmp(page,"exit"))
  {
    printf("exit - Exit tish, return to AMS\n\n");
    printf("exit\n");
  }
  else if (!strcmp(page,"help"))
  {
    printf("help - show built in help for tish\n\n");
    printf("help [topic]\n\n");
    printf("Example: help help (show this text)\n");
  }
  else if (!strcmp(page,"ls"))
  {
    printf("ls - List files and folders\n\n");
    printf("ls [-h] [-l] [foldername]\n\n");
    printf("  -h - list home (folders)\n");
    printf("  -l - long form listing with details\n\n");
    printf("foldername is current folder if omitted\n");
  }
  else if (!strcmp(page,"mkdir"))
  {
    printf("mkdir - Create directory (folder)\n\n");
    printf("mkdir <foldername>\n");
  }
  else if (!strcmp(page,"mv"))
  {
    printf("mv - Move/rename file\n\n");
    printf("mv <source> <destination>\n\n");
    printf("Note: mv cannot move archived files\n");
  }
  else if (!strcmp(page,"rm"))
  {
    printf("rm - Delete file\n\n");
    printf("rm <filename>\n\n");
    printf("Note: rm cannot delete archived files\n");
  }
  else if (!strcmp(page,"rmdir"))
  {
    printf("rmdir - Delete directory (folder)\n\n");
    printf("rmdir <foldername>\n\n");
    printf("Warning: this command will delete not\n");
    printf("only the folder but also every file in\n");
    printf("the folder, even locked/archived files\n");
  }
  else if (!strcmp(page,"off"))
  {
    printf("off - Power off calculator\n\n");
    printf("off\n");
  }
  else if (!strcmp(page,"pause"))
  {
    printf("pause - Wait and display text message\n\n");
    printf("pause [text]\n");
  }
  else if (!strcmp(page,"unarc"))
  {
    printf("unarc - move file from archive to RAM\n\n");
    printf("unarc <filename>\n");
  }
  else
  {
    printf("No help for that topic\n");
  }
}

