/*  -----------------------------------------------------------------------
    tish - unixlike shell & tools for the TI-92+ and Voyage 200 calculators
    Copyright (C) 2020 Adam Colley <mog@kupo.be>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
------------------------------------------------------------------------ */

// tish shell - 0.01 - manticore
// this is the main part of tish
 
#define USE_TI92PLUS
#define USE_V200
#define SAVE_SCREEN
 
// comment out if ghost space execution is needed (which
// it shouldn't be, use hw3patch to fix your AMS code!)
// #define NO_GHOST
 
#include <tigcclib.h>

// and now for the prototypes 
void cls();
void updatestatus();
void readline(char *, char *, unsigned short);
void docommand(char *);
int progexec(char *);
int scriptexec(char *);
int gottts();
void poweroff();
int wildmatch(char *, char *);
 
// and we're off. with an additional _. for some reason.
void _main (void)
{
  char buf[50];
  char obuf[50];
  ESI argptr = top_estack;
  SYM_ENTRY *SymPtr;
    
  cls();
  updatestatus();
  printf("tish 0.01 (manticore) <mog@kupo.be>\n\n");
  printf("tish comes with ABSOLUTELY NO WARRANTY\n");
  printf("This is free software, and you are\n");
  printf("welcome to redistribute it under\n");
  printf("certain conditions; see the included\n");
  printf("license.txt for the full GNU GPL2 text\n\n");  
  printf("Type 'help' for help, 'exit' to exit\n\n");

  // execute any commands passed as arguments
  while (GetArgType(top_estack)!=END_TAG)
  {
    char *buf = GetStrnArg (argptr);
    top_estack = next_expression_index (top_estack);
    docommand(buf);
  }

  // check for and execute if present autoexec
  SymPtr = DerefSym (SymFind (SYMSTR ("tish\\autoexec")));
  if (SymPtr)
    progexec("tish\\autoexec");
 
  // enter main loop
  do
  {
    printf(">");
    readline(buf,obuf,36);
    if (strcmp(buf,"exit"))
      docommand(buf);
  }
  while (strcmp(buf,"exit"));
}

int scriptexec(char *fname)
{
  SYM_ENTRY *SymPtr;
  HANDLE h;
  unsigned short plen;
  char *cptr;
  char *cptrpos;
  char *cptrmark;
  char linebuf[48];
  
  SymPtr = DerefSym (SymFind (SYMSTR (fname)));
  h=SymPtr->handle;
  cptr=HeapDeref(h);
  plen=*(short*)(cptr)+3;
  cptrpos=cptr+5;
  
  while (cptrpos<cptr+plen-3)
  {
    cptrmark=strchr(cptrpos,13);
    if (cptrmark)
    {
      if (cptrmark-cptrpos<40)
      {
        strncpy(linebuf,cptrpos,cptrmark-cptrpos);
        linebuf[cptrmark-cptrpos]=0;
        docommand(linebuf);
        cptrpos=cptrmark+2;
      }
      else
      {
        printf("Line too long in script, aborted\n");
        return 1;
      }
    }
    else
    {
      printf("Last line not terminated, ignored\n");
      return 1;
    }
  }
}

void updatestatus(void)
{
  char sbuf[80];
  char fbuf[10];
  unsigned long inuse;
  unsigned long freeaftergc;
  unsigned long freeb;

  EM_survey(&inuse,&freeaftergc,&freeb,NULL,NULL,NULL);
  FolderGetCur(fbuf);
  sprintf(sbuf,"[tish]   RAM free: %ld   Arc free: %ld   Folder: %s",HeapAvail(),freeb+freeaftergc,fbuf);  
  ST_helpMsg(sbuf);
}
 
void poweroff(void)
{
  #ifdef __GTC__
  asm
  {
    trap #4
  };
  #else
  asm
  ("
    trap #4
  ");
  #endif
}

void cls(void)
{
  FontSetSys(F_6x8);
  ClrScr();
  DrawLine(0,121,239,121,A_NORMAL);
  MoveTo(0,0);
  updatestatus();
}

void docommand(char *buf)
{
  char tempstr[40];
  int ret;

  // catch incorrect attempts to get the folder list
  if ((!strcmp(buf,"ls \\")) || (!strcmp(buf,"ls /")))
     sprintf(buf,"ls -h");

  if (!strncmp(buf,"#",1))
  {
    // It's a comment, do nothing
  }
  else if (!strcmp(buf,"off"))
  {
    poweroff();
  }
  else if (!strncmp(buf,"echo",4))
  {
    if (strlen(buf)<=5)
      printf("\n");
    else
      printf("%s\n",&buf[5]);
  }
  else if (!strncmp(buf,"pause",5))
  {
    if (strlen(buf)<=6)
      printf("Press any key to continue...");
    else
      printf("%s",&buf[6]);
    GKeyIn(0,0);
    printf("\n");
  }
  else if (!strncmp(buf,"cd",2))
  {
    if (strlen(buf)<=3)
    {
      printf ("Assuming 'cd main' intended\n");
      strcpy(buf,"cd main");
    }
    if (!FolderCur(SYMSTR(&buf[3]),TRUE))
      printf("No such folder\n");
    updatestatus();
  }
  else if (!strcmp(buf,"clear"))
  {
    cls();
  }
  else if (strlen(buf)>0)
  {
    ret=progexec(buf);
    if (ret==1)
    {
      sprintf(tempstr,"tish\\%s",buf);
      ret=progexec(tempstr);
      if (ret==1)
      {
        sprintf(tempstr,"bin\\%s",buf);
        ret=progexec(tempstr);
      }
    }
   if (ret==1)
     printf("File not found\n");
   if (ret==2)
     printf("Out of memory\n");
   if (ret==3)
     printf("Cannot execute file\n");
  }
}
 
// really rewrite this to allow scrolling single line editing up to 255 characters 
void readline(char *buf, char *obuf, unsigned short maxlen)
{
  SCR_STATE ss;
  short key;
  unsigned short i=0;
  buf[0]=0;

  SaveScrState(&ss);
  do
  {
    MoveTo(ss.CurX,ss.CurY);
    printf("%s_ ",buf);
    key=GKeyIn(0,0);
    if (key==KEY_OFF)
      poweroff();
    updatestatus();
    if (key>=' ' && key<='}' && i<maxlen)
      buf[i++]=key;
    else if ((key==KEY_BACKSPACE || key==KEY_LEFT) && i)
      i--;
	else if ((key==KEY_F1 || key==KEY_RIGHT) && (strlen(obuf)>i))
	{
	  buf[i]=obuf[i];
	  i++;
	}
	else if ((key==KEY_F3 || key==KEY_UP) && (strlen(obuf)>=i))
	{
	  strcpy(buf,obuf);
	  i=strlen(obuf);
	}
    buf[i]=0;
  }
  while (key != KEY_ENTER);
  MoveTo(ss.CurX,ss.CurY);
  printf("%s \n",buf);
  strcpy(obuf,buf);
}

int gottts(void)
{  
  SYM_ENTRY *SymPtr;
 
  SymPtr = DerefSym (SymFind (SYMSTR ("tish\\ttstart")));
  if (!SymPtr)
    return FALSE;
  return TRUE;
}   

// fuck me, this is an absolute mess. works however.
int progexec(char *filename)
{  
  char *fptr, *cptr;
  unsigned short plens;
  char ffname[48];
  char objname[48];
  char *parms,*parmi;
  unsigned char ftyp;
  int isexec;
  signed long plen;
  ESI argptr = top_estack;
  SYM_ENTRY *SymPtr;
  HANDLE h;

  parms=strchr(filename,' ');
  if (parms)
  {
    strncpy(ffname,filename,parms-filename);
    ffname[parms-filename]=0;
    parms++;
  } 
  else
    strcpy(ffname,filename);

  strcpy(objname,ffname);

  SymPtr = DerefSym (SymFind (SYMSTR (ffname)));
  if (!SymPtr) 
  {
    // file not found
    return 1;
  }

  // check for type here. jesus fucking christ.
  h = SymPtr->handle;
  if (HeapGetLock (h))
  {
    cptr = fptr = HeapDeref (h);
    h = 0;
  }
  else
  {
    cptr = fptr = HLock (h);
  }
  plens = *(short*)(cptr) + 3;
  // this is a workaround for a gtc bug
  // where unsigned shorts are signed.
  plen = plens;
  if (plen<0)
    plen=plen+65536;
  ftyp=cptr[plen-2];
  switch (ftyp)
  {
    case 0xf3:
      isexec=1;
      if (!strncmp(&cptr[6],"68kP",4))
      {
        if (gottts())
          isexec=2;
        else
          isexec=0;
      }
      if (!strncmp(&cptr[6],"68kL",4))
        isexec=0;
      break;
       
    case 0xf8:
      isexec=0;
      if ((!strncmp(&cptr[plen-6],"ppg",3)) && (gottts()))
        isexec=2;
      break;
      
    case 0xe0:
      isexec=0;
      if (!strncmp(&cptr[5],"#tish",5))
        isexec=3;
      break;
 
    default:
      isexec=0;
  }
  HeapUnlock(h);
  
  if (isexec==0)
    return 3; // invalid executable type / no ttstart
  if (isexec==2)
  {
    sprintf(ffname,"tish\\ttstart");
    SymPtr = DerefSym (SymFind (SYMSTR (ffname)));
  }
  if (isexec==3)
  {
    // it's a tish script, not a binary
    scriptexec(objname);
    return 0;
  }

  h = SymPtr->handle;
  if (HeapGetLock (h))
  {
    cptr = fptr = HeapDeref (h);
    h = 0;
  }
  else
  {
    cptr = fptr = HLock (h);
  }
  plen = *(short*)(cptr) + 3;

  if (SymPtr->flags.bits.archived)
  {
    if (!(cptr = malloc (plen)))
    {
      if (h) HeapUnlock (h);
      {
        // out of memory
        return 2;
      }
    }
    memcpy (cptr, fptr, plen);
  }
  // We're going for it so deal with passing parameters
  if (parms)
  {
    while (strrchr(parms,' '))
    {
      parmi=strrchr(parms,' ');
      push_zstr(&parmi[1]);
      parmi[0]=0;
    }
    push_zstr(parms);
  }
  if (isexec==2)
    push_zstr(objname);
  #ifdef NO_GHOST
  EX_patch (cptr + 2, cptr + plen - 2);
  ASM_call (cptr+2);
  #else
  enter_ghost_space ();
  EX_patch (cptr + 0x40002, cptr + plen + 0x3FFFE);
  ASM_call (cptr + 0x40002);
  #endif
  if (h) HeapUnlock (h);
  if (cptr != fptr) free (cptr);
  FontSetSys(F_6x8);
  // clean any leftover shit off the estack
  while (GetArgType(top_estack)!=END_TAG)
  {
    top_estack = next_expression_index (top_estack);
  }
  return 0;
}

// Check for wildcard match. first string may be wild.
int wildmatch(char *first, char *second) 
{ 
  if (*first == '\0' && *second == '\0') 
    return TRUE; 
  if (*first == '*' && *(first+1) != '\0' && *second == '\0') 
    return FALSE; 
  if (*first == '?' || *first == *second) 
    return wildmatch(first+1, second+1); 
  if (*first == '*') 
    return wildmatch(first+1, second) || wildmatch(first, second+1); 
  return FALSE; 
} 
